@@ -1,7 +1,13 @@
-# SPDX-License-Identifier: GPL-3.0-or-later
+# ******************************************************************************
 #
 # pccc, the Python Conventional Commit Checker.
-# Copyright (C) 2020-2021 Jeremy A Gray <jeremy.a.gray@gmail.com>.
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+#
+# Copyright 2020-2022 Jeremy A Gray <gray@flyquackswim.com>.
+#
+# ******************************************************************************
+
 """pccc configuration functions."""
 
 import argparse
@@ -20,51 +26,52 @@ class Config:
     """Class for accessing and loading pccc configuration options.
 
     Provides default values for all pccc configuration options and
-    loading methods for reading pyproject.toml and CLI options.
+    loading methods for reading ``pyproject.toml`` and CLI options.
 
     Attributes
     ----------
-    commit : string
-        Commit message location; default is ``STDIN``.
-    config_file : string
-        Configuration file path; default is ``pyproject.toml``.
-    header_length : int
-       Maximum header length; default is 50.
-    body_length : int
-       Maximum body line length; default is 72.
-    repair : boolean
-        Repair commit, implying spell check and rewrap; default is
-        ``False``.
-    rewrap : boolean
-        Rewrap body; default is ``False``.
-    spell_check : boolean
-        Spell check header and body; default is ``False``.
-    ignore_generated_commits : boolean
+    commit : string, default="STDIN"
+        Commit message location.
+    config_file : string, default="pyproject.toml"
+        Configuration file path.
+    header_length : int, default=50
+       Maximum header length.
+    body_length : int, default=72
+       Maximum body line length.
+    repair : boolean, default=False
+        Repair commit, implying spell check and wrap.
+    wrap : boolean, default=False
+        Wrap body and breaking change, if necessary.
+    force_wrap : boolean, default=False
+        Force wrap body and breaking change, regardless of length.
+    spell_check : boolean, default=False
+        Spell check commit message.
+    ignore_generated_commits : boolean, default=False
         Ignore generated commits which match ``generated_commits``
-        regular expressions; default is ``False``.
-    generated_commits : [string]
-        List of generated commits, as Python regular expressions;
-        default is ``[]``.  For TOML files, it's probably best to use
-        the multiline single quote strings to reduce escaping
-        problems.
-    types : [string]
-        List of header types; default is ``['feat', 'fix']``.
-    scopes : [string]
-        List of header scopes; default is ``[]``.
-    footers : [string]
-        List of footers; default is ``[]``.
-    required_footers : [string]
-        List of required footers; default is ``[]``.
+        regular expressions.
+    generated_commits : iterable of string, default=[]
+        List of generated commits, as Python regular expressions.  For
+        TOML files, it's probably best to use the multiline single
+        quote strings to reduce escaping problems.
+    types : iterable of string, default=["feat", "fix"]
+        List of header types.
+    scopes : iterable of string, default=[]
+        List of header scopes.
+    footers : iterable of string, default=[]
+        List of footers.
+    required_footers : iterable of string, default=[]
+        List of required footers.
     """
 
     def __init__(
         self,
         commit="",
-        config_file="./pyproject.toml",
+        config_file="pyproject.toml",
         header_length=50,
         body_length=72,
         repair=False,
-        rewrap=False,
+        wrap=False,
+        force_wrap=False,
         spell_check=False,
         ignore_generated_commits=False,
         generated_commits=[],
@@ -88,7 +95,8 @@ class Config:
         self.header_length = header_length
         self.body_length = body_length
         self.repair = repair
-        self.rewrap = rewrap
+        self.wrap = wrap
+        self.force_wrap = force_wrap
         self.spell_check = spell_check
         self.ignore_generated_commits = ignore_generated_commits
         self.generated_commits = generated_commits
@@ -124,7 +132,7 @@ class Config:
             rs = json.dumps({"pccc": self.config_as_dict()}, indent=2)
         else:
             # TOML, by default.
-            rs = toml.dumps({"tool": {"pccc": self.config_as_dict()}})
+            rs = toml.dumps({"pccc": self.config_as_dict()})
 
         return rs
 
@@ -136,7 +144,8 @@ class Config:
             f"header_length={self.header_length}, "
             f"body_length={self.body_length}, "
             f"repair={self.repair}, "
-            f"rewrap={self.rewrap}, "
+            f"wrap={self.wrap}, "
+            f"force_wrap={self.force_wrap}, "
             f"spell_check={self.spell_check}, "
             f"ignore_generated_commits={self.ignore_generated_commits}, "
             f"generated_commits={self.generated_commits}, "
@@ -177,7 +186,8 @@ class Config:
             "header_length": self.header_length,
             "body_length": self.body_length,
             "repair": self.repair,
-            "rewrap": self.rewrap,
+            "wrap": self.wrap,
+            "force_wrap": self.force_wrap,
             "spell_check": self.spell_check,
             "ignore_generated_commits": self.ignore_generated_commits,
             "generated_commits": self.generated_commits,
@@ -238,10 +248,6 @@ class Config:
         later values overriding previous values.
 
         Unset values are explicitly ``None`` at each level.
-
-        Handles any ``FileNotFound``, ``JSONDecodeError``, or
-        ``TomlDecodeError`` exceptions that arise during loading of
-        configuration file by ignoring the file.
         """
         # Parse the CLI options to make configuration file path available.
         args = _create_argument_parser().parse_args(argv)
@@ -250,39 +256,95 @@ class Config:
         if args.config_file is not None:
             self.config_file = args.config_file
 
-        try:
-            self.update(**_load_file(self.config_file))
-        except (FileNotFoundError,):
-            print(
-                f"Unable to find configuration file {self.config_file},"
-                " using defaults and CLI options."
-            )
-        except (json.JSONDecodeError,):
-            print(
-                f"Unable to parse configuration file {self.config_file}"
-                " (default package.json), using defaults and CLI options.\n"
-                "Ensure that file format matches extension."
-            )
-        except (toml.TomlDecodeError,):
-            print(
-                f"Unable to parse configuration file {self.config_file}"
-                " (default pyproject.toml), using defaults and CLI options."
-                "  Ensure that file format matches extension."
-            )
+        # Load the configuration file values.
+        self.update(**_load_file(self.config_file))
 
+        # Load the CLI configuration values.
         self.update(**vars(args))
 
         return
 
 
-def _load_file(filename="./pyproject.toml"):
-    """Load a configuration file, using the ``[tool.pccc]`` section.
+def _determine_file_format(filename):
+    """Determine the format of a configuration file.
 
-    Load a ``pyproject.toml`` configuration file, using the
-    ``[tool.pccc]`` section, or a ``package.json`` configuration file,
-    using the ``pccc`` entry.  Will only load ``package.json`` if
-    ``pyproject.toml`` is not available or if ``package.json`` is
-    explicitly set as the configuration file.
+    Determine the format of a configuration file by attempting to
+    parse the file's contents as TOML, JSON, YAML, and finally BespON,
+    in that order.  Returns the format or raises ``ValueError`` on
+    failure to determine the format.
+
+    Parameters
+    ----------
+    filename : string
+        Configuration file name.
+
+    Returns
+    -------
+    format : string
+       The file format, one of ``toml``, ``json``, ``yaml``, or
+       ``bespon``.
+
+    Raises
+    ------
+    ValueError
+        Raised if the format of the file does not appear to match any
+        of the defined formats.
+    FileNotFoundError
+        Raised if the configuration file does not exist or is not
+        readable.
+    """
+    contents = ""
+    try:
+        with open(filename, "r") as file:
+            contents = file.read()
+    except FileNotFoundError as error:
+        print(f"{error.strerror}: {error.filename}")
+        raise
+
+    try:
+        toml.loads(contents)
+        return "toml"
+    except toml.TomlDecodeError:
+        pass
+
+    try:
+        json.loads(contents)
+        return "json"
+    except json.JSONDecodeError:
+        pass
+
+    # Not implemented.
+    # try:
+    #     yaml.loads(contents)
+    #     return "yaml"
+    # except yaml.YamlDecodeError:
+    #     pass
+
+    # Not implemented.
+    # try:
+    #     bespon.loads(contents)
+    #     return "bespon"
+    # except bespon.BespONDecodeError:
+    #     pass
+
+    raise ValueError("Configuration file format not recognized")
+
+
+def _load_file(filename=None):
+    """Load a configuration file.
+
+    Load a configuration file, defaulting to the first loadable file
+    of ``pyproject.toml``, ``pccc.toml``, ``package.json``,
+    ``pccc.json``, ``pccc.yaml``, ``pccc.yml``, and finally
+    ``pccc.besp``.  If a file is specified, it will be prepended to
+    the default list and processed identically.  The first file that
+    exists and loads will be used.  If no usable file is found,
+    default configuration values will be used.
+
+    Note that this loader does not associate file formats and
+    extensions; if a file format is recognized and is different than
+    that purported by the extension, it will still load (i.e. it will
+    load TOML from ``package.json``, etc.).
 
     Parameters
     ----------
@@ -299,54 +361,49 @@ def _load_file(filename="./pyproject.toml"):
     ------
     JSONDecodeError
         Raised if there are problems decoding a JSON configuration
-        file.
+        file, but should only be raised if the file changes between
+        format detection and loading.
     TomlDecodeError
         Raised if there are problems decoding a TOML configuration
-        file.
+        file, but should only be raised if the file changes between
+        format detection and loading.
     FileNotFoundError
         Raised if the configuration file does not exist or is not
-        readable.
+        readable, but should only be raised if the file changes
+        between format detection and loading.
     """
     options = {}
-    jsonRE = re.compile(r"^.*\.json$", re.IGNORECASE)
-    tomlRE = re.compile(r"^.*\.toml$", re.IGNORECASE)
+    files = (
+        "pyproject.toml",
+        "pccc.toml",
+        "package.json",
+        "pccc.json",
+        "pccc.yaml",
+        "pccc.yml",
+        "pccc.besp",
+    )
+    if filename:
+        files = (filename,) + files
 
-    if os.path.abspath(filename) == os.path.abspath("./pyproject.toml"):
+    for file in files:
         try:
-            # Default to ``./pyproject.toml``.
-            options = _load_toml_file(filename)
-        except toml.TomlDecodeError:
-            raise
-        except FileNotFoundError:
-            # No file, bail.
-            raise
-    elif jsonRE.match(filename):
-        try:
-            # Well, if JSON is supplied, use it.
-            options = _load_json_file(filename)
-        except json.JSONDecodeError:
-            raise
-        except FileNotFoundError:
-            raise
-    elif tomlRE.match(filename):
-        try:
-            options = _load_toml_file(filename)
-        except toml.TomlDecodeError:
-            raise
-        except FileNotFoundError:
-            raise
-    else:
-        try:
-            # Last chance, parse filename as TOML.
-            options = _load_toml_file(filename)
-        except toml.TomlDecodeError:
-            # Okay, one last try.  Maybe it was JSON.
-            try:
-                options = _load_json_file(filename)
-            except json.JSONDecodeError:
-                raise NotParseableError(filename)
-        except FileNotFoundError:
-            raise
+            format = _determine_file_format(file)
+        except (
+            FileNotFoundError,
+            ValueError,
+        ) as error:
+            print(error)
+            continue
+
+        if format == "toml":
+            return _load_toml_file(file)
+        elif format == "json":
+            return _load_json_file(file)
+        # Not implemented.
+        # elif format == "yaml":
+        #     return _load_yaml_file(file)
+        # elif format == "bespon":
+        #     return _load_bespon_file(file)
 
     return options
 
@@ -400,7 +457,8 @@ def _load_json_file(filename="./package.json"):
         "body_length": None,
         "spell_check": None,
         "ignore_generated_commits": None,
-        "rewrap": None,
+        "wrap": None,
+        "force_wrap": None,
         "repair": None,
         "types": None,
         "scopes": None,
@@ -416,15 +474,16 @@ def _load_json_file(filename="./package.json"):
     return empty_options
 
 
-def _load_toml_file(filename="./pyproject.toml"):
-    """Load a toml configuration file.
+def _load_toml_file(filename="pyproject.toml"):
+    """Load a TOML configuration file.
 
-    Load a ``pyproject.toml`` configuration file, returning the
-    ``[tool.pccc]`` section.
+    Load a TOML configuration file, returning the ``[pccc]`` section.
+    If the file is ``pyproject.toml``, then the ``[tool.pccc]``
+    section is returned.
 
     Parameters
     ----------
-    filename : string (optional)
+    filename : string (optional), default = "pyproject.toml"
         Configuration file to load.
 
     Returns
@@ -456,7 +515,6 @@ def _load_toml_file(filename="./pyproject.toml"):
         raise
     except FileNotFoundError as error:
         print(f"{error.strerror}: {error.filename}")
-        print("trying package.json...")
         raise
 
     empty_options = {
@@ -465,7 +523,8 @@ def _load_toml_file(filename="./pyproject.toml"):
         "header_length": None,
         "body_length": None,
         "spell_check": None,
-        "rewrap": None,
+        "wrap": None,
+        "force_wrap": None,
         "repair": None,
         "types": None,
         "scopes": None,
@@ -475,12 +534,74 @@ def _load_toml_file(filename="./pyproject.toml"):
         "generated_commits": None,
     }
 
-    for (k, v) in config["tool"]["pccc"].items():
-        empty_options[k] = v
+    if "pyproject.toml" in filename:
+        for (k, v) in config["tool"]["pccc"].items():
+            empty_options[k] = v
+    else:
+        for (k, v) in config["pccc"].items():
+            empty_options[k] = v
 
     return empty_options
 
 
+def _load_yaml_file(filename="./pccc.yaml"):
+    """Load a YAML configuration file, using the ``pccc`` entry.
+
+    Load a YAML configuration file, returning options from the
+    top-level ``pccc`` dictionary.
+
+    Parameters
+    ----------
+    filename : string (optional), default="pccc.yaml"
+        Configuration file to load.
+
+    Returns
+    -------
+    dict : options
+       Configuration option keys and values, with unset values
+       explicitly set to ``None``.
+
+    Raises
+    ------
+    YamlDecodeError
+        Raised if there are problems decoding a YAML configuration
+        file.
+    FileNotFoundError
+        Raised if the configuration file does not exist or is not
+        readable.
+    """
+    raise NotImplementedError
+
+
+def _load_bespon_file(filename="./pccc.besp"):
+    """Load a BespON configuration file, using the ``pccc`` entry.
+
+    Load a BespON configuration file, returning options from the
+    top-level ``pccc`` dictionary.
+
+    Parameters
+    ----------
+    filename : string (optional), default="pccc.besp"
+        Configuration file to load.
+
+    Returns
+    -------
+    dict : options
+       Configuration option keys and values, with unset values
+       explicitly set to ``None``.
+
+    Raises
+    ------
+    BesponDecodeError
+        Raised if there are problems decoding a BespON configuration
+        file.
+    FileNotFoundError
+        Raised if the configuration file does not exist or is not
+        readable.
+    """
+    raise NotImplementedError
+
+
 def _create_argument_parser():
     """Create an argparse argument parser."""
     parser = argparse.ArgumentParser(
@@ -519,8 +640,8 @@ to redistribute it under certain conditions; type ``pccc
         "--config-file",
         dest="config_file",
         type=str,
-        default="./pyproject.toml",
-        help="Path to configuration file.  Default is ./pyproject.toml.",
+        default="pyproject.toml",
+        help="Path to configuration file.  Default is ``pyproject.toml``.",
     )
 
     parser.add_argument(
@@ -583,24 +704,41 @@ to redistribute it under certain conditions; type ``pccc
         " ``generated_commits``.  Default is to check every commit.",
     )
 
-    rewrap_group = parser.add_mutually_exclusive_group()
-    rewrap_group.add_argument(
+    wrap_group = parser.add_mutually_exclusive_group()
+    wrap_group.add_argument(
         "-w",
-        "--rewrap",
-        dest="rewrap",
+        "--wrap",
+        dest="wrap",
         default=None,
         action="store_true",
-        help="Rewrap the body commit, regardless of line length."
-        "  Default is no rewrapping.",
+        help="Wrap the body commit, if necessary." "  Default is no wrapping.",
     )
-    rewrap_group.add_argument(
+    wrap_group.add_argument(
         "-W",
-        "--no-rewrap",
-        dest="rewrap",
+        "--no-wrap",
+        dest="wrap",
         default=None,
         action="store_false",
-        help="Do not rewrap the body commit, regardless of line length."
-        "  Default is no rewrapping.",
+        help="Do not wrap the body commit." "  Default is no wrapping.",
+    )
+
+    force_wrap_group = parser.add_mutually_exclusive_group()
+    force_wrap_group.add_argument(
+        "-z",
+        "--force-wrap",
+        dest="force_wrap",
+        default=None,
+        action="store_true",
+        help="Wrap the body commit, regardless of line length."
+        "  Default is no force wrapping.",
+    )
+    force_wrap_group.add_argument(
+        "-Z",
+        "--no-force-wrap",
+        dest="force_wrap",
+        default=None,
+        action="store_false",
+        help="Do not wrap the body commit." "  Default is no force wrapping.",
     )
 
     repair_group = parser.add_mutually_exclusive_group()
@@ -610,7 +748,7 @@ to redistribute it under certain conditions; type ``pccc
         dest="repair",
         default=None,
         action="store_true",
-        help="Repair the body commit as necessary; implies spell check and rewrap."
+        help="Repair the body commit as necessary; implies spell check and wrap."
         "  Default is false.",
     )
     repair_group.add_argument(
@@ -620,7 +758,7 @@ to redistribute it under certain conditions; type ``pccc
         default=None,
         action="store_false",
         help="Do not repair the body commit; implies no spell check and no"
-        " rewrap.  Default is false.",
+        " wrap.  Default is false.",
     )
 
     parser.add_argument(
@@ -683,7 +821,7 @@ class _ShowLicenseAction(argparse.Action):
         license = """\
 pccc:  The Python Conventional Commit Checker.
 
-Copyright (C) 2021 Jeremy A Gray <jeremy.a.gray@gmail.com>.
+Copyright (C) 2021-2022 Jeremy A Gray <jeremy.a.gray@gmail.com>.
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
